You are viewing a plain text version of this content. The canonical link for it is here.
Posted to commits@camel.apache.org by da...@apache.org on 2016/04/01 13:56:20 UTC

[2/6] camel git commit: CAMEL-9759: camel-zipkin - Instrument Camel. Work in progress.

CAMEL-9759: camel-zipkin - Instrument Camel. Work in progress.


Project: http://git-wip-us.apache.org/repos/asf/camel/repo
Commit: http://git-wip-us.apache.org/repos/asf/camel/commit/4dc5b5e4
Tree: http://git-wip-us.apache.org/repos/asf/camel/tree/4dc5b5e4
Diff: http://git-wip-us.apache.org/repos/asf/camel/diff/4dc5b5e4

Branch: refs/heads/master
Commit: 4dc5b5e484381655e407c590239d4ed90a36b705
Parents: 0729332
Author: Claus Ibsen <da...@apache.org>
Authored: Fri Apr 1 06:42:15 2016 +0200
Committer: Claus Ibsen <da...@apache.org>
Committed: Fri Apr 1 13:55:01 2016 +0200

----------------------------------------------------------------------
 .../zipkin/starter/ZipkinAutoConfiguration.java |   7 +-
 .../starter/ZipkinConfigurationProperties.java  |  26 +-
 components/camel-zipkin/pom.xml                 |   4 +-
 .../camel/zipkin/ZipkinEventNotifier.java       | 305 ++++++++++++++-----
 .../org/apache/camel/zipkin/ZipkinState.java    |  65 ++++
 .../camel/zipkin/ZipkinRouteConcurrentTest.java |   4 +-
 .../camel/zipkin/ZipkinSimpleRouteTest.java     |   2 +-
 .../apache/camel/zipkin/ZipkinTwoRouteTest.java |   6 +-
 .../scribe/ZipkinAutoConfigureScribe.java       |   3 +-
 .../zipkin/scribe/ZipkinOneRouteScribe.java     |   3 +-
 .../scribe/ZipkinRouteConcurrentScribe.java     |  86 ------
 .../zipkin/scribe/ZipkinSimpleRouteScribe.java  |   3 +-
 .../zipkin/scribe/ZipkinTimerRouteScribe.java   |   3 +-
 .../zipkin/scribe/ZipkinTwoRouteScribe.java     |   6 +-
 examples/camel-example-zipkin/README.md         |  39 ++-
 examples/camel-example-zipkin/pom.xml           | 105 ++-----
 examples/camel-example-zipkin/service1/pom.xml  | 113 +++++++
 .../src/main/java/sample/camel/CounterBean.java |  32 ++
 .../java/sample/camel/Service1Application.java  |  41 +++
 .../main/java/sample/camel/Service1Route.java   |  47 +++
 .../src/main/resources/META-INF/LICENSE.txt     | 203 ++++++++++++
 .../src/main/resources/META-INF/NOTICE.txt      |  11 +
 .../src/main/resources/META-INF/beans.xml       |  18 ++
 .../src/main/resources/log4j.properties         |  28 ++
 examples/camel-example-zipkin/service2/pom.xml  |  90 ++++++
 .../java/sample/camel/Service2Application.java  |  42 +++
 .../main/java/sample/camel/Service2Route.java   |  33 ++
 .../src/main/resources/META-INF/LICENSE.txt     | 203 ++++++++++++
 .../src/main/resources/META-INF/NOTICE.txt      |  11 +
 .../src/main/resources/application.properties   |  37 +++
 examples/camel-example-zipkin/service3/pom.xml  |  95 ++++++
 .../java/sample/camel/Service3Application.java  |  29 ++
 .../main/java/sample/camel/Service3Route.java   |  40 +++
 .../src/main/resources/META-INF/LICENSE.txt     | 203 ++++++++++++
 .../src/main/resources/META-INF/NOTICE.txt      |  11 +
 .../src/main/resources/log4j.properties         |  28 ++
 .../src/main/java/sample/camel/HelloBean.java   |  38 ---
 .../java/sample/camel/HelloCamelRouter.java     |  40 ---
 .../java/sample/camel/ReplyCamelRouter.java     |  39 ---
 .../sample/camel/ZipkinCamelApplication.java    |  42 ---
 .../src/main/resources/application.properties   |  47 ---
 41 files changed, 1693 insertions(+), 495 deletions(-)
----------------------------------------------------------------------


http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinAutoConfiguration.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinAutoConfiguration.java b/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinAutoConfiguration.java
index 0466d34..5f750a8 100644
--- a/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinAutoConfiguration.java
+++ b/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinAutoConfiguration.java
@@ -46,8 +46,11 @@ public class ZipkinAutoConfiguration {
         if (config.getExcludePatterns() != null) {
             notifier.setExcludePatterns(config.getExcludePatterns());
         }
-        if (config.getServiceMappings() != null) {
-            notifier.setServiceMappings(config.getServiceMappings());
+        if (config.getClientServiceMappings() != null) {
+            notifier.setClientServiceMappings(config.getClientServiceMappings());
+        }
+        if (config.getServerServiceMappings() != null) {
+            notifier.setServerServiceMappings(config.getServerServiceMappings());
         }
         notifier.setIncludeMessageBody(config.isIncludeMessageBody());
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinConfigurationProperties.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/pom.xml b/components/camel-zipkin/pom.xml
index 9473ef2..7ae9d26 100644
--- a/components/camel-zipkin/pom.xml
+++ b/components/camel-zipkin/pom.xml
@@ -77,7 +77,7 @@
           <!-- needed for testing auto configure -->
           <environmentVariables>
             <ZIPKIN_SERVICE_HOST>192.168.99.100</ZIPKIN_SERVICE_HOST>
-            <ZIPKIN_SERVICE_PORT>9401</ZIPKIN_SERVICE_PORT>
+            <ZIPKIN_SERVICE_PORT>9410</ZIPKIN_SERVICE_PORT>
           </environmentVariables>
         </configuration>
       </plugin>
@@ -98,7 +98,7 @@
               <reuseForks>true</reuseForks>
               <forkedProcessTimeoutInSeconds>300</forkedProcessTimeoutInSeconds>
               <includes>
-                <include>**/*ScribeTest.java</include>
+                <include>**/*Scribe.java</include>
               </includes>
             </configuration>
           </plugin>

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinEventNotifier.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinEventNotifier.java b/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinEventNotifier.java
index 3d91c6f..f8e11ff 100644
--- a/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinEventNotifier.java
+++ b/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinEventNotifier.java
@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- *
- *      http://www.apache.org/licenses/LICENSE-2.0
- *
+ * <p/>
+ * http://www.apache.org/licenses/LICENSE-2.0
+ * <p/>
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -87,7 +87,8 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
     private int port;
     private float rate = 1.0f;
     private SpanCollector spanCollector;
-    private Map<String, String> serviceMappings = new HashMap<>();
+    private Map<String, String> clientServiceMappings = new HashMap<>();
+    private Map<String, String> serverServiceMappings = new HashMap<>();
     private Set<String> excludePatterns = new HashSet<>();
     private Map<String, Brave> braves = new HashMap<>();
     private boolean includeMessageBody;
@@ -152,33 +153,53 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
     }
 
     public String getServiceName() {
-        return serviceMappings.get("*");
+        return clientServiceMappings.get("*");
     }
 
     /**
      * To use a global service name that matches all Camel events
      */
     public void setServiceName(String serviceName) {
-        serviceMappings.put("*", serviceName);
+        clientServiceMappings.put("*", serviceName);
+        serverServiceMappings.put("*", serviceName);
     }
 
-    public Map<String, String> getServiceMappings() {
-        return serviceMappings;
+    public Map<String, String> getClientServiceMappings() {
+        return clientServiceMappings;
     }
 
-    public void setServiceMappings(Map<String, String> serviceMappings) {
-        this.serviceMappings = serviceMappings;
+    public void setClientServiceMappings(Map<String, String> clientServiceMappings) {
+        this.clientServiceMappings = clientServiceMappings;
     }
 
     /**
-     * Adds a service mapping that matches Camel events to the given zipkin service name.
+     * Adds a client service mapping that matches Camel events to the given zipkin service name.
      * See more details at the class javadoc.
      *
      * @param pattern  the pattern such as route id, endpoint url
      * @param serviceName the zipkin service name
      */
-    public void addServiceMapping(String pattern, String serviceName) {
-        serviceMappings.put(pattern, serviceName);
+    public void addClientServiceMapping(String pattern, String serviceName) {
+        clientServiceMappings.put(pattern, serviceName);
+    }
+
+    public Map<String, String> getServerServiceMappings() {
+        return serverServiceMappings;
+    }
+
+    public void setServerServiceMappings(Map<String, String> serverServiceMappings) {
+        this.serverServiceMappings = serverServiceMappings;
+    }
+
+    /**
+     * Adds a server service mapping that matches Camel events to the given zipkin service name.
+     * See more details at the class javadoc.
+     *
+     * @param pattern  the pattern such as route id, endpoint url
+     * @param serviceName the zipkin service name
+     */
+    public void addServerServiceMapping(String pattern, String serviceName) {
+        serverServiceMappings.put(pattern, serviceName);
     }
 
     public Set<String> getExcludePatterns() {
@@ -238,13 +259,13 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
 
         ObjectHelper.notNull(spanCollector, "SpanCollector", this);
 
-        if (serviceMappings.isEmpty()) {
+        if (clientServiceMappings.isEmpty()) {
             log.warn("No service name(s) has been configured. Camel will fallback and use endpoint uris as service names.");
             useFallbackServiceNames = true;
         }
 
         // create braves mapped per service name
-        for (Map.Entry<String, String> entry : serviceMappings.entrySet()) {
+        for (Map.Entry<String, String> entry : clientServiceMappings.entrySet()) {
             String pattern = entry.getKey();
             String serviceName = entry.getValue();
             Brave brave = braves.get(pattern);
@@ -282,7 +303,17 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
                 || event instanceof ExchangeFailedEvent;
     }
 
-    private String getServiceName(Exchange exchange, Endpoint endpoint) {
+    private String getServiceName(EventObject event, Exchange exchange, Endpoint endpoint, boolean server, boolean client) {
+        if (client) {
+            return getServiceName(event, exchange, endpoint, clientServiceMappings, server, client);
+        } else if (server) {
+            return getServiceName(event, exchange, endpoint, serverServiceMappings, server, client);
+        } else {
+            return null;
+        }
+    }
+
+    private String getServiceName(EventObject event, Exchange exchange, Endpoint endpoint, Map<String, String> serviceMappings, boolean server, boolean client) {
         String answer = null;
 
         // endpoint takes precedence over route
@@ -305,20 +336,23 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
             }
         }
 
-        if (answer == null && exchange.getFromEndpoint() != null) {
-            String url = exchange.getFromEndpoint().getEndpointUri();
-            if (url != null) {
-                // exclude patterns take precedence
-                for (String pattern : excludePatterns) {
-                    if (EndpointHelper.matchEndpoint(exchange.getContext(), url, pattern)) {
-                        return null;
+        // special for created event as its a server side to know where it was from
+        if (server && (event instanceof ExchangeCreatedEvent || event instanceof ExchangeCompletedEvent || event instanceof ExchangeFailedEvent)) {
+            if (answer == null && exchange.getFromEndpoint() != null) {
+                String url = exchange.getFromEndpoint().getEndpointUri();
+                if (url != null) {
+                    // exclude patterns take precedence
+                    for (String pattern : excludePatterns) {
+                        if (EndpointHelper.matchEndpoint(exchange.getContext(), url, pattern)) {
+                            return null;
+                        }
                     }
-                }
-                for (Map.Entry<String, String> entry : serviceMappings.entrySet()) {
-                    String pattern = entry.getKey();
-                    if (EndpointHelper.matchEndpoint(exchange.getContext(), url, pattern)) {
-                        answer = entry.getValue();
-                        break;
+                    for (Map.Entry<String, String> entry : serviceMappings.entrySet()) {
+                        String pattern = entry.getKey();
+                        if (EndpointHelper.matchEndpoint(exchange.getContext(), url, pattern)) {
+                            answer = entry.getValue();
+                            break;
+                        }
                     }
                 }
             }
@@ -410,37 +444,46 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
 
     @Override
     public void notify(EventObject event) throws Exception {
+        // client events
         if (event instanceof ExchangeSendingEvent) {
             ExchangeSendingEvent ese = (ExchangeSendingEvent) event;
-            String serviceName = getServiceName(ese.getExchange(), ese.getEndpoint());
+            String serviceName = getServiceName(ese, ese.getExchange(), ese.getEndpoint(), false, true);
             Brave brave = getBrave(serviceName);
             if (brave != null) {
                 clientRequest(brave, serviceName, ese);
             }
         } else if (event instanceof ExchangeSentEvent) {
             ExchangeSentEvent ese = (ExchangeSentEvent) event;
-            String serviceName = getServiceName(ese.getExchange(), ese.getEndpoint());
+            String serviceName = getServiceName(ese, ese.getExchange(), ese.getEndpoint(), false, true);
             Brave brave = getBrave(serviceName);
             if (brave != null) {
                 clientResponse(brave, serviceName, ese);
             }
-        } else if (event instanceof ExchangeCreatedEvent) {
+        }
+
+        // server received client request
+        if (event instanceof ExchangeCreatedEvent) {
             ExchangeCreatedEvent ece = (ExchangeCreatedEvent) event;
-            String serviceName = getServiceName(ece.getExchange(), null);
-            Brave brave = getBrave(serviceName);
-            if (brave != null) {
-                serverRequest(brave, serviceName, ece);
+            // we should only emit a server request if we have an existing trace id (eg a client has called us)
+            if (hasZipkinTraceId(ece.getExchange())) {
+                String serviceName = getServiceName(ece, ece.getExchange(), null, true, false);
+                Brave brave = getBrave(serviceName);
+                if (brave != null) {
+                    serverRequest(brave, serviceName, ece);
+                }
             }
-        } else if (event instanceof ExchangeCompletedEvent) {
+        }
+        // server completed events
+        if (event instanceof ExchangeCompletedEvent) {
             ExchangeCompletedEvent ece = (ExchangeCompletedEvent) event;
-            String serviceName = getServiceName(ece.getExchange(), null);
+            String serviceName = getServiceName(ece, ece.getExchange(), null, true, false);
             Brave brave = getBrave(serviceName);
             if (brave != null) {
                 serverResponse(brave, serviceName, ece);
             }
         } else if (event instanceof ExchangeFailedEvent) {
             ExchangeFailedEvent efe = (ExchangeFailedEvent) event;
-            String serviceName = getServiceName(efe.getExchange(), null);
+            String serviceName = getServiceName(efe, efe.getExchange(), null, true, false);
             Brave brave = getBrave(serviceName);
             if (brave != null) {
                 serverResponse(brave, serviceName, efe);
@@ -450,86 +493,180 @@ public class ZipkinEventNotifier extends EventNotifierSupport implements Statefu
 
     private void clientRequest(Brave brave, String serviceName, ExchangeSendingEvent event) {
         ClientSpanThreadBinder binder = brave.clientSpanThreadBinder();
+
+        // reuse existing span if we do multiple requests from the same
+        ZipkinState state = event.getExchange().getProperty(ZipkinState.KEY, ZipkinState.class);
+        if (state == null) {
+            state = new ZipkinState();
+            event.getExchange().setProperty(ZipkinState.KEY, state);
+        }
+        Object last = state.getLast();
+        if (last != null && last instanceof Span) {
+            binder.setCurrentSpan((Span) last);
+        } else if (last != null && last instanceof ServerSpan) {
+            Span span = ((ServerSpan) last).getSpan();
+            binder.setCurrentSpan(span);
+        }
+
+        brave.
+
         brave.clientRequestInterceptor().handle(new ZipkinClientRequestAdapter(this, serviceName, event.getExchange(), event.getEndpoint()));
-        Span span = binder.getCurrentClientSpan();
 
-        String key = "CamelZipkinClientSpan-" + serviceName;
-        event.getExchange().setProperty(key, span);
+        // store span after request
+        Span span = binder.getCurrentClientSpan();
+        state.pushClientSpan(span);
+        // and reset binder
+        binder.setCurrentSpan(null);
 
         if (log.isDebugEnabled()) {
-            String id = "<null>";
+            String traceId = "<null>";
             if (span != null) {
-                id = "" + span.getId();
+                traceId = "" + span.getTrace_id();
             }
-            log.debug("clientRequest\t[service={}, spanId={}]", serviceName, id);
+            String spanId = "<null>";
+            if (span != null) {
+                spanId = "" + span.getId();
+            }
+            String parentId = "<null>";
+            if (span != null) {
+                parentId = "" + span.getParent_id();
+            }
+            log.debug("clientRequest [service={}, traceId={}, spanId={}, parentId={}]", serviceName, traceId, spanId, parentId);
         }
     }
 
     private void clientResponse(Brave brave, String serviceName, ExchangeSentEvent event) {
-        ClientSpanThreadBinder binder = brave.clientSpanThreadBinder();
-        String key = "CamelZipkinClientSpan-" + serviceName;
-        Span span = event.getExchange().getProperty(key, Span.class);
-        binder.setCurrentSpan(span);
-        brave.clientResponseInterceptor().handle(new ZipkinClientResponseAdaptor(this, event.getExchange(), event.getEndpoint()));
-        binder.setCurrentSpan(null);
+        Span span = null;
+        ZipkinState state = event.getExchange().getProperty(ZipkinState.KEY, ZipkinState.class);
+        if (state != null) {
+            span = state.popClientSpan();
+        }
 
-        if (log.isDebugEnabled()) {
-            String id = "<null>";
-            if (span != null) {
-                id = "" + span.getId();
+        if (span != null) {
+            // only process if it was a zipkin client event
+            ClientSpanThreadBinder binder = brave.clientSpanThreadBinder();
+            binder.setCurrentSpan(span);
+            brave.clientResponseInterceptor().handle(new ZipkinClientResponseAdaptor(this, event.getExchange(), event.getEndpoint()));
+            // and reset binder
+            binder.setCurrentSpan(null);
+
+            if (log.isDebugEnabled()) {
+                String traceId = "" + span.getTrace_id();
+                String spanId = "" + span.getId();
+                String parentId = "" + span.getParent_id();
+                log.debug("clientResponse[service={}, traceId={}, spanId={}, parentId={}]", serviceName, traceId, spanId, parentId);
             }
-            log.debug("clientResponse\t[service={}, spanId={}]", serviceName, id);
         }
     }
 
     private void serverRequest(Brave brave, String serviceName, ExchangeCreatedEvent event) {
         ServerSpanThreadBinder binder = brave.serverSpanThreadBinder();
+
+        // reuse existing span if we do multiple requests from the same
+        ZipkinState state = event.getExchange().getProperty(ZipkinState.KEY, ZipkinState.class);
+        if (state == null) {
+            state = new ZipkinState();
+            event.getExchange().setProperty(ZipkinState.KEY, state);
+        }
+        Object last = state.getLast();
+        if (last != null && last instanceof ServerSpan) {
+            binder.setCurrentSpan((ServerSpan) last);
+        }
+
         brave.serverRequestInterceptor().handle(new ZipkinServerRequestAdapter(this, event.getExchange()));
+
+        // store span after request
         ServerSpan span = binder.getCurrentServerSpan();
-        String key = "CamelZipkinServerSpan-" + serviceName;
-        event.getExchange().setProperty(key, span);
+        state.pushServerSpan(span);
+        // and reset binder
+        binder.setCurrentSpan(null);
 
         if (log.isDebugEnabled()) {
-            String id = "<null>";
-            if (span != null && span.getSpan() != null) {
-                id = "" + span.getSpan().getId();
+            String traceId = "<null>";
+            if (span.getSpan() != null) {
+                traceId = "" + span.getSpan().getTrace_id();
+            }
+            String spanId = "<null>";
+            if (span.getSpan() != null) {
+                spanId = "" + span.getSpan().getId();
+            }
+            String parentId = "<null>";
+            if (span.getSpan() != null) {
+                parentId = "" + span.getSpan().getParent_id();
             }
-            log.debug("serverRequest\t[service={}, spanId={}]", serviceName, id);
+            log.debug("serverRequest [service={}, traceId={}, spanId={}, parentId={}]", serviceName, traceId, spanId, parentId);
         }
     }
 
     private void serverResponse(Brave brave, String serviceName, ExchangeCompletedEvent event) {
-        ServerSpanThreadBinder binder = brave.serverSpanThreadBinder();
-        String key = "CamelZipkinServerSpan-" + serviceName;
-        ServerSpan span = event.getExchange().getProperty(key, ServerSpan.class);
-        binder.setCurrentSpan(span);
-        brave.serverResponseInterceptor().handle(new ZipkinServerResponseAdapter(this, event.getExchange()));
-        binder.setCurrentSpan(null);
+        ServerSpan span = null;
+        ZipkinState state = event.getExchange().getProperty(ZipkinState.KEY, ZipkinState.class);
+        if (state != null) {
+            span = state.popServerSpan();
+        }
 
-        if (log.isDebugEnabled()) {
-            String id = "<null>";
-            if (span != null && span.getSpan() != null) {
-                id = "" + span.getSpan().getId();
+        if (span != null) {
+            // only process if it was a zipkin server event
+            ServerSpanThreadBinder binder = brave.serverSpanThreadBinder();
+            binder.setCurrentSpan(span);
+            brave.serverResponseInterceptor().handle(new ZipkinServerResponseAdapter(this, event.getExchange()));
+            // and reset binder
+            binder.setCurrentSpan(null);
+
+            if (log.isDebugEnabled()) {
+                String traceId = "<null>";
+                if (span.getSpan() != null) {
+                    traceId = "" + span.getSpan().getTrace_id();
+                }
+                String spanId = "<null>";
+                if (span.getSpan() != null) {
+                    spanId = "" + span.getSpan().getId();
+                }
+                String parentId = "<null>";
+                if (span.getSpan() != null) {
+                    parentId = "" + span.getSpan().getParent_id();
+                }
+                log.debug("serverResponse[service={}, traceId={}, spanId={}, parentId={}] [status=exchangeFailed]", serviceName, traceId, spanId, parentId);
             }
-            log.debug("serverResponse\t[service={}, spanId={}]\t[status=exchangeCompleted]", serviceName, id);
         }
     }
 
     private void serverResponse(Brave brave, String serviceName, ExchangeFailedEvent event) {
-        ServerSpanThreadBinder binder = brave.serverSpanThreadBinder();
-        String key = "CamelZipkinServerSpan-" + serviceName;
-        ServerSpan span = event.getExchange().getProperty(key, ServerSpan.class);
-        binder.setCurrentSpan(span);
-        brave.serverResponseInterceptor().handle(new ZipkinServerResponseAdapter(this, event.getExchange()));
-        binder.setCurrentSpan(null);
+        ServerSpan span = null;
+        ZipkinState state = event.getExchange().getProperty(ZipkinState.KEY, ZipkinState.class);
+        if (state != null) {
+            span = state.popServerSpan();
+        }
 
-        if (log.isDebugEnabled()) {
-            String id = "<null>";
-            if (span != null && span.getSpan() != null) {
-                id = "" + span.getSpan().getId();
+        if (span != null) {
+            // only process if it was a zipkin server event
+            ServerSpanThreadBinder binder = brave.serverSpanThreadBinder();
+            binder.setCurrentSpan(span);
+            brave.serverResponseInterceptor().handle(new ZipkinServerResponseAdapter(this, event.getExchange()));
+            // and reset binder
+            binder.setCurrentSpan(null);
+
+            if (log.isDebugEnabled()) {
+                String traceId = "<null>";
+                if (span.getSpan() != null) {
+                    traceId = "" + span.getSpan().getTrace_id();
+                }
+                String spanId = "<null>";
+                if (span.getSpan() != null) {
+                    spanId = "" + span.getSpan().getId();
+                }
+                String parentId = "<null>";
+                if (span.getSpan() != null) {
+                    parentId = "" + span.getSpan().getParent_id();
+                }
+                log.debug("serverResponse[service={}, traceId={}, spanId={}, parentId={}] [status=exchangeFailed]", serviceName, traceId, spanId, parentId);
             }
-            log.debug("serverResponse[service={}, spanId={}]\t[status=exchangeFailed]", serviceName, id);
         }
     }
 
+    private boolean hasZipkinTraceId(Exchange exchange) {
+        // must have zipkin headers to start a server event
+        return exchange.getIn().getHeader(ZipkinConstants.TRACE_ID) != null;
+    }
+
 }

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinState.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinState.java b/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinState.java
new file mode 100644
index 0000000..d09af34
--- /dev/null
+++ b/components/camel-zipkin/src/main/java/org/apache/camel/zipkin/ZipkinState.java
@@ -0,0 +1,65 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * <p/>
+ * http://www.apache.org/licenses/LICENSE-2.0
+ * <p/>
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package org.apache.camel.zipkin;
+
+import java.util.Stack;
+
+import com.github.kristofa.brave.ServerSpan;
+import com.twitter.zipkin.gen.Span;
+
+public class ZipkinState {
+
+    public static final String KEY = "CamelZipkinState";
+
+    private final Stack<Span> clientSpans = new Stack<>();
+    private final Stack<ServerSpan> serverSpans = new Stack<>();
+    private Object last;
+
+    public void pushClientSpan(Span span) {
+        clientSpans.push(span);
+        last = span;
+    }
+
+    public Span popClientSpan() {
+        if (!clientSpans.empty()) {
+            Span answer = clientSpans.pop();
+            last = answer;
+            return answer;
+        } else {
+            return null;
+        }
+    }
+
+    public void pushServerSpan(ServerSpan span) {
+        serverSpans.push(span);
+        last = span;
+    }
+
+    public ServerSpan popServerSpan() {
+        if (!serverSpans.empty()) {
+            ServerSpan answer = serverSpans.pop();
+            last = answer;
+            return answer;
+        } else {
+            return null;
+        }
+    }
+
+    public Object getLast() {
+        return last;
+    }
+}

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinRouteConcurrentTest.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinRouteConcurrentTest.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinRouteConcurrentTest.java
index ac607de..987e188 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinRouteConcurrentTest.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinRouteConcurrentTest.java
@@ -34,8 +34,8 @@ public class ZipkinRouteConcurrentTest extends CamelTestSupport {
         CamelContext context = super.createCamelContext();
 
         zipkin = new ZipkinEventNotifier();
-        zipkin.addServiceMapping("seda:foo", "foo");
-        zipkin.addServiceMapping("seda:bar", "bar");
+        zipkin.addClientServiceMapping("seda:foo", "foo");
+        zipkin.addServerServiceMapping("seda:bar", "bar");
         zipkin.setSpanCollector(new ZipkinLoggingSpanCollector());
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinSimpleRouteTest.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinSimpleRouteTest.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinSimpleRouteTest.java
index 2114ea0..8edabe6 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinSimpleRouteTest.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinSimpleRouteTest.java
@@ -34,7 +34,7 @@ public class ZipkinSimpleRouteTest extends CamelTestSupport {
         CamelContext context = super.createCamelContext();
 
         zipkin = new ZipkinEventNotifier();
-        zipkin.addServiceMapping("seda:dude", "dude");
+        zipkin.setServiceName("dude");
         zipkin.setSpanCollector(new ZipkinLoggingSpanCollector());
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinTwoRouteTest.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinTwoRouteTest.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinTwoRouteTest.java
index c7bdb9c..5819562 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinTwoRouteTest.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/ZipkinTwoRouteTest.java
@@ -32,8 +32,10 @@ public class ZipkinTwoRouteTest extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have 2 routes as services
-        zipkin.addServiceMapping("seda:cat", "cat");
-        zipkin.addServiceMapping("seda:dog", "dog");
+        zipkin.addClientServiceMapping("seda:cat", "cat");
+        zipkin.addServerServiceMapping("seda:cat", "cat");
+        zipkin.addClientServiceMapping("seda:dog", "dog");
+        zipkin.addServerServiceMapping("seda:dog", "dog");
         // capture message body as well
         zipkin.setIncludeMessageBody(true);
         zipkin.setSpanCollector(new ZipkinLoggingSpanCollector());

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinAutoConfigureScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinAutoConfigureScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinAutoConfigureScribe.java
index db70c62..a569af7 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinAutoConfigureScribe.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinAutoConfigureScribe.java
@@ -41,7 +41,8 @@ public class ZipkinAutoConfigureScribe extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have one route as service
-        zipkin.addServiceMapping("seda:cat", "cat");
+        zipkin.addClientServiceMapping("seda:cat", "cat");
+        zipkin.addServerServiceMapping("seda:cat", "cat");
         // should auto configure as we have not setup a spanCollector
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinOneRouteScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinOneRouteScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinOneRouteScribe.java
index 93da37b..2f9fe37 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinOneRouteScribe.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinOneRouteScribe.java
@@ -43,7 +43,8 @@ public class ZipkinOneRouteScribe extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have one route as service
-        zipkin.addServiceMapping("seda:cat", "cat");
+        zipkin.addClientServiceMapping("seda:cat", "cat");
+        zipkin.addServerServiceMapping("seda:cat", "cat");
         zipkin.setSpanCollector(new ScribeSpanCollector(ip, 9410));
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinRouteConcurrentScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinRouteConcurrentScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinRouteConcurrentScribe.java
deleted file mode 100644
index 527d7a2..0000000
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinRouteConcurrentScribe.java
+++ /dev/null
@@ -1,86 +0,0 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- *
- *      http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
-package org.apache.camel.zipkin.scribe;
-
-import java.util.concurrent.TimeUnit;
-
-import com.github.kristofa.brave.scribe.ScribeSpanCollector;
-import org.apache.camel.CamelContext;
-import org.apache.camel.RoutesBuilder;
-import org.apache.camel.builder.NotifyBuilder;
-import org.apache.camel.builder.RouteBuilder;
-import org.apache.camel.test.junit4.CamelTestSupport;
-import org.apache.camel.zipkin.ZipkinEventNotifier;
-import org.junit.Test;
-
-/**
- * Integration test requires running Zipkin/Scribe running
- *
- * The easiest way is to run using zipkin-docker: https://github.com/openzipkin/docker-zipkin
- *
- * Adjust the IP address to what IP docker-machines have assigned, you can use
- * <tt>docker-machines ls</tt>
- */
-public class ZipkinRouteConcurrentScribe extends CamelTestSupport {
-
-    private String ip = "192.168.99.100";
-    private ZipkinEventNotifier zipkin;
-
-    @Override
-    protected CamelContext createCamelContext() throws Exception {
-        CamelContext context = super.createCamelContext();
-
-        zipkin = new ZipkinEventNotifier();
-        // we have 2 routes as services
-        zipkin.addServiceMapping("seda:foo", "foo");
-        zipkin.addServiceMapping("seda:bar", "bar");
-        zipkin.setSpanCollector(new ScribeSpanCollector(ip, 9410));
-        context.getManagementStrategy().addEventNotifier(zipkin);
-
-        return context;
-    }
-
-    @Test
-    public void testZipkinRoute() throws Exception {
-        NotifyBuilder notify = new NotifyBuilder(context).from("seda:foo").whenDone(5).create();
-
-        for (int i = 0; i < 5; i++) {
-            template.requestBody("direct:foo", "Hello World");
-        }
-
-        assertTrue(notify.matches(60, TimeUnit.SECONDS));
-    }
-
-    @Override
-    protected RoutesBuilder createRouteBuilder() throws Exception {
-        return new RouteBuilder() {
-            @Override
-            public void configure() throws Exception {
-                from("direct:start").to("sed:foo");
-
-                from("seda:foo?concurrentConsumers=2").routeId("foo")
-                        .log("routing at ${routeId}")
-                        .delay(simple("${random(1000,2000)}"))
-                        .to("seda:bar");
-
-                from("seda:bar?concurrentConsumers=2").routeId("bar")
-                        .log("routing at ${routeId}")
-                        .delay(simple("${random(0,500)}"));
-            }
-        };
-    }
-}

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinSimpleRouteScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinSimpleRouteScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinSimpleRouteScribe.java
index 7848967..e3d176d 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinSimpleRouteScribe.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinSimpleRouteScribe.java
@@ -43,7 +43,8 @@ public class ZipkinSimpleRouteScribe extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have one route as service
-        zipkin.addServiceMapping("seda:dude", "dude");
+        zipkin.addClientServiceMapping("seda:dude", "dude");
+        zipkin.addServerServiceMapping("seda:dude", "dude");
         zipkin.setSpanCollector(new ScribeSpanCollector(ip, 9410));
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTimerRouteScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTimerRouteScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTimerRouteScribe.java
index 237df63..7e0ba4b 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTimerRouteScribe.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTimerRouteScribe.java
@@ -47,7 +47,8 @@ public class ZipkinTimerRouteScribe extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have one route as service
-        zipkin.addServiceMapping("seda:timer", "timer");
+        zipkin.addClientServiceMapping("seda:timer", "timer");
+        zipkin.addServerServiceMapping("seda:timer", "timer");
         zipkin.setSpanCollector(new ScribeSpanCollector(ip, 9410));
         context.getManagementStrategy().addEventNotifier(zipkin);
 

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTwoRouteScribe.java
----------------------------------------------------------------------
diff --git a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTwoRouteScribe.java b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTwoRouteScribe.java
index fc55e40..b5e4c6d 100644
--- a/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTwoRouteScribe.java
+++ b/components/camel-zipkin/src/test/java/org/apache/camel/zipkin/scribe/ZipkinTwoRouteScribe.java
@@ -43,8 +43,10 @@ public class ZipkinTwoRouteScribe extends CamelTestSupport {
 
         zipkin = new ZipkinEventNotifier();
         // we have 2 routes as services
-        zipkin.addServiceMapping("seda:cat", "cat");
-        zipkin.addServiceMapping("seda:dog", "dog");
+        zipkin.addClientServiceMapping("seda:cat", "cat");
+        zipkin.addServerServiceMapping("seda:cat", "cat");
+        zipkin.addClientServiceMapping("seda:dog", "dog");
+        zipkin.addServerServiceMapping("seda:dog", "dog");
         // capture message body as well
         zipkin.setIncludeMessageBody(true);
         zipkin.setSpanCollector(new ScribeSpanCollector(ip, 9410));

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/README.md
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/README.md b/examples/camel-example-zipkin/README.md
index 11320b2..a1e9ee3 100644
--- a/examples/camel-example-zipkin/README.md
+++ b/examples/camel-example-zipkin/README.md
@@ -6,12 +6,17 @@ This example shows how to use Camel with Zipkin to trace/timing all incoming and
 
 The example requires a running Zipkin Server.
 
-The example includes two routes. The client route uses a timer to send a message to the server route.
-The server route then replies back to the client.
+The example includes two sub maven modules that implement
+
+- service1
+- service2
+
+Where service1 calls service2 using HTTP.
 
 ### Configuration
 
-The example is configured in the `src/main/resources/application.properties` file.
+Service1 is configured in the `src/main/java/sample/camel/Service1Application.java` source code.
+Service2 is configured in the `src/main/resources/application.properties` properties file.
 
 Here you need to configure the hostname and port number for the Zipkin Server.
 
@@ -23,10 +28,26 @@ You will need to compile this example first:
 $ mvn compile
 ```
 
+### Run the example
+
+Then using two different shells and run service2 before service1.
+
+```sh
+$ cd service2
+$ mvn compile spring-boot:run
+```
+
+When service2 is ready then start service1
+
+```sh
+$ cd service1
+$ mvn compile camel:run
+```
+
 ### Zipkin web console
 
 You should be able to visualize the traces and timings from this example using the Zipkin Web Console.
-The service name is named `hello`.
+The services are named `service1` and `service2`.
 
 
 ### Installing Zipkin Server using Docker
@@ -38,16 +59,6 @@ Remember to configure the IP address and port number in the `application.propert
 
 You can find the IP using `docker-machine ls`
 
-### Run
-
-To run the example, type:
-
-```sh
-$ mvn spring-boot:run
-```
-
-To stop the example hit <kbd>ctrl</kbd>+<kbd>c</kbd>.
-
 ### Forum, Help, etc
 
 If you hit an problems please let us know on the Camel Forums

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/pom.xml
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/pom.xml b/examples/camel-example-zipkin/pom.xml
index 6159295..0017648 100644
--- a/examples/camel-example-zipkin/pom.xml
+++ b/examples/camel-example-zipkin/pom.xml
@@ -1,102 +1,39 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!--
-	Licensed to the Apache Software Foundation (ASF) under one or more
-	contributor license agreements. See the NOTICE file distributed with
-	this work for additional information regarding copyright ownership.
-	The ASF licenses this file to You under the Apache License, Version
-	2.0 (the "License"); you may not use this file except in compliance
-	with the License. You may obtain a copy of the License at
-
-	http://www.apache.org/licenses/LICENSE-2.0 Unless required by
-	applicable law or agreed to in writing, software distributed under the
-	License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-	CONDITIONS OF ANY KIND, either express or implied. See the License for
-	the specific language governing permissions and limitations under the
-	License.
+    Licensed to the Apache Software Foundation (ASF) under one or more
+    contributor license agreements. See the NOTICE file distributed with
+    this work for additional information regarding copyright ownership.
+    The ASF licenses this file to You under the Apache License, Version
+    2.0 (the "License"); you may not use this file except in compliance
+    with the License. You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0 Unless required by
+    applicable law or agreed to in writing, software distributed under the
+    License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
+    CONDITIONS OF ANY KIND, either express or implied. See the License for
+    the specific language governing permissions and limitations under the
+    License.
 -->
 <project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
          xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/maven-v4_0_0.xsd">
 
   <modelVersion>4.0.0</modelVersion>
 
- <parent>
+  <parent>
     <groupId>org.apache.camel</groupId>
     <artifactId>examples</artifactId>
     <version>2.18-SNAPSHOT</version>
   </parent>
-  
+
   <artifactId>camel-example-zipkin</artifactId>
+  <packaging>pom</packaging>
   <name>Camel :: Example :: Zipkin</name>
   <description>An example showing how to trace incoming and outgoing messages from Camel with Zipkin</description>
 
-  <properties>
-    <spring.boot-version>${spring-boot-version}</spring.boot-version>
-  </properties>
-
-  <!-- import Spring-Boot and Camel BOM -->
-  <dependencyManagement>
-    <dependencies>
-      <dependency>
-        <groupId>org.springframework.boot</groupId>
-        <artifactId>spring-boot-dependencies</artifactId>
-        <version>${spring.boot-version}</version>
-        <type>pom</type>
-        <scope>import</scope>
-      </dependency>
-      <dependency>
-        <groupId>org.apache.camel</groupId>
-        <artifactId>camel-parent</artifactId>
-        <version>${project.version}</version>
-        <type>pom</type>
-        <scope>import</scope>
-      </dependency>
-    </dependencies>
-  </dependencyManagement>
-
-  <dependencies>
-
-    <!-- spring-boot -->
-    <dependency>
-      <groupId>org.springframework.boot</groupId>
-      <artifactId>spring-boot-starter</artifactId>
-    </dependency>
-
-    <!-- camel -->
-    <dependency>
-      <groupId>org.apache.camel</groupId>
-      <artifactId>camel-spring-boot-starter</artifactId>
-    </dependency>
-    <dependency>
-      <groupId>org.apache.camel</groupId>
-      <artifactId>camel-zipkin-starter</artifactId>
-    </dependency>
-    <dependency>
-      <groupId>org.apache.camel</groupId>
-      <artifactId>camel-stream</artifactId>
-    </dependency>
-
-    <!-- test -->
-    <dependency>
-      <groupId>org.springframework.boot</groupId>
-      <artifactId>spring-boot-starter-test</artifactId>
-      <scope>test</scope>
-    </dependency>
-    <dependency>
-      <groupId>org.apache.camel</groupId>
-      <artifactId>camel-test</artifactId>
-      <scope>test</scope>
-    </dependency>
-
-  </dependencies>
-
-  <build>
-    <plugins>
-      <plugin>
-        <groupId>org.springframework.boot</groupId>
-        <artifactId>spring-boot-maven-plugin</artifactId>
-        <version>${spring-boot-version}</version>
-      </plugin>
-    </plugins>
-  </build>
+  <modules>
+    <module>service1</module>
+    <module>service2</module>
+    <module>service3</module>
+  </modules>
 
 </project>

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/pom.xml
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/pom.xml b/examples/camel-example-zipkin/service1/pom.xml
new file mode 100644
index 0000000..092fee9
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/pom.xml
@@ -0,0 +1,113 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!--
+	Licensed to the Apache Software Foundation (ASF) under one or more
+	contributor license agreements. See the NOTICE file distributed with
+	this work for additional information regarding copyright ownership.
+	The ASF licenses this file to You under the Apache License, Version
+	2.0 (the "License"); you may not use this file except in compliance
+	with the License. You may obtain a copy of the License at
+
+	http://www.apache.org/licenses/LICENSE-2.0 Unless required by
+	applicable law or agreed to in writing, software distributed under the
+	License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
+	CONDITIONS OF ANY KIND, either express or implied. See the License for
+	the specific language governing permissions and limitations under the
+	License.
+-->
+<project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/maven-v4_0_0.xsd">
+
+  <modelVersion>4.0.0</modelVersion>
+
+  <parent>
+    <groupId>org.apache.camel</groupId>
+    <artifactId>camel-example-zipkin</artifactId>
+    <version>2.18-SNAPSHOT</version>
+  </parent>
+
+  <artifactId>camel-example-zipkin-service1</artifactId>
+  <name>Camel :: Example :: Zipkin :: Service 1</name>
+  <description>An example showing how to trace incoming and outgoing messages from Camel with Zipkin</description>
+
+  <properties>
+    <spring.boot-version>${spring-boot-version}</spring.boot-version>
+  </properties>
+
+  <!-- import Camel BOM -->
+  <dependencyManagement>
+    <dependencies>
+      <dependency>
+        <groupId>org.apache.camel</groupId>
+        <artifactId>camel-parent</artifactId>
+        <version>${project.version}</version>
+        <type>pom</type>
+        <scope>import</scope>
+      </dependency>
+    </dependencies>
+  </dependencyManagement>
+
+  <dependencies>
+
+    <!-- CDI API -->
+    <dependency>
+      <groupId>javax.enterprise</groupId>
+      <artifactId>cdi-api</artifactId>
+      <version>${cdi-api-1.2-version}</version>
+      <scope>provided</scope>
+    </dependency>
+
+    <!-- camel-cdi -->
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-cdi</artifactId>
+    </dependency>
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-zipkin</artifactId>
+    </dependency>
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-http</artifactId>
+    </dependency>
+
+    <!-- logging -->
+
+
+  </dependencies>
+
+  <build>
+
+    <plugins>
+      <!-- allows the routes to be run via 'mvn camel:run' -->
+      <plugin>
+        <groupId>org.apache.camel</groupId>
+        <artifactId>camel-maven-plugin</artifactId>
+        <version>${project.version}</version>
+        <dependencies>
+          <dependency>
+            <groupId>org.apache.deltaspike.cdictrl</groupId>
+            <artifactId>deltaspike-cdictrl-weld</artifactId>
+            <version>${deltaspike-version}</version>
+          </dependency>
+          <dependency>
+            <groupId>org.jboss.weld.se</groupId>
+            <artifactId>weld-se</artifactId>
+            <version>${weld2-version}</version>
+          </dependency>
+          <dependency>
+            <groupId>org.slf4j</groupId>
+            <artifactId>slf4j-log4j12</artifactId>
+            <version>${slf4j-version}</version>
+          </dependency>
+          <dependency>
+            <groupId>log4j</groupId>
+            <artifactId>log4j</artifactId>
+            <version>${log4j-version}</version>
+          </dependency>
+        </dependencies>
+      </plugin>
+    </plugins>
+
+  </build>
+
+</project>

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/java/sample/camel/CounterBean.java
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/java/sample/camel/CounterBean.java b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/CounterBean.java
new file mode 100644
index 0000000..0c27215
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/CounterBean.java
@@ -0,0 +1,32 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package sample.camel;
+
+import javax.inject.Named;
+import javax.inject.Singleton;
+
+@Singleton
+@Named("counterBean")
+public class CounterBean {
+
+    private int counter;
+
+    public String someMethod(String body) {
+        return "" + ++counter;
+    }
+
+}

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Application.java
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Application.java b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Application.java
new file mode 100644
index 0000000..4bbd8a3
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Application.java
@@ -0,0 +1,41 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * <p/>
+ * http://www.apache.org/licenses/LICENSE-2.0
+ * <p/>
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package sample.camel;
+
+import javax.enterprise.event.Observes;
+
+import org.apache.camel.cdi.ContextName;
+import org.apache.camel.management.event.CamelContextStartingEvent;
+import org.apache.camel.zipkin.ZipkinEventNotifier;
+
+@ContextName("Server1")
+public class Service1Application {
+
+    public void setupCamel(@Observes CamelContextStartingEvent event) {
+        // create zipkin
+        ZipkinEventNotifier zipkin = new ZipkinEventNotifier();
+        zipkin.setHostName("192.168.99.100");
+        zipkin.setPort(9410);
+        zipkin.setServiceName("service1");
+        // do not track timer as its an internal event to just get an event
+        zipkin.addExcludePattern("timer:*");
+
+        // add zipkin to CamelContext
+        event.getContext().getManagementStrategy().addEventNotifier(zipkin);
+    }
+
+}

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Route.java
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Route.java b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Route.java
new file mode 100644
index 0000000..7e1be20
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/java/sample/camel/Service1Route.java
@@ -0,0 +1,47 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * <p/>
+ * http://www.apache.org/licenses/LICENSE-2.0
+ * <p/>
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package sample.camel;
+
+import org.apache.camel.builder.RouteBuilder;
+import org.apache.camel.zipkin.ZipkinEventNotifier;
+
+public class Service1Route extends RouteBuilder {
+
+    @Override
+    public void configure() {
+        // you can configure the route rule with Java DSL here
+
+        // TODO: use CDI to setup Camel instead of here in the route
+
+        // create zipkin
+        ZipkinEventNotifier zipkin = new ZipkinEventNotifier();
+        zipkin.setHostName("192.168.99.100");
+        zipkin.setPort(9410);
+        zipkin.addClientServiceMapping("http://localhost:9090/service2", "service2");
+        zipkin.addClientServiceMapping("http://localhost:7070/service3", "service3");
+
+        // add zipkin to CamelContext
+        getContext().getManagementStrategy().addEventNotifier(zipkin);
+
+        from("timer:trigger?exchangePattern=InOut&period=30s")
+            .bean("counterBean")
+            .to("http://localhost:9090/service2")
+            .to("http://localhost:7070/service3")
+            .log("Result: ${body}");
+    }
+
+}

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/resources/META-INF/LICENSE.txt
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/resources/META-INF/LICENSE.txt b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/LICENSE.txt
new file mode 100644
index 0000000..6b0b127
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/LICENSE.txt
@@ -0,0 +1,203 @@
+
+                                 Apache License
+                           Version 2.0, January 2004
+                        http://www.apache.org/licenses/
+
+   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
+
+   1. Definitions.
+
+      "License" shall mean the terms and conditions for use, reproduction,
+      and distribution as defined by Sections 1 through 9 of this document.
+
+      "Licensor" shall mean the copyright owner or entity authorized by
+      the copyright owner that is granting the License.
+
+      "Legal Entity" shall mean the union of the acting entity and all
+      other entities that control, are controlled by, or are under common
+      control with that entity. For the purposes of this definition,
+      "control" means (i) the power, direct or indirect, to cause the
+      direction or management of such entity, whether by contract or
+      otherwise, or (ii) ownership of fifty percent (50%) or more of the
+      outstanding shares, or (iii) beneficial ownership of such entity.
+
+      "You" (or "Your") shall mean an individual or Legal Entity
+      exercising permissions granted by this License.
+
+      "Source" form shall mean the preferred form for making modifications,
+      including but not limited to software source code, documentation
+      source, and configuration files.
+
+      "Object" form shall mean any form resulting from mechanical
+      transformation or translation of a Source form, including but
+      not limited to compiled object code, generated documentation,
+      and conversions to other media types.
+
+      "Work" shall mean the work of authorship, whether in Source or
+      Object form, made available under the License, as indicated by a
+      copyright notice that is included in or attached to the work
+      (an example is provided in the Appendix below).
+
+      "Derivative Works" shall mean any work, whether in Source or Object
+      form, that is based on (or derived from) the Work and for which the
+      editorial revisions, annotations, elaborations, or other modifications
+      represent, as a whole, an original work of authorship. For the purposes
+      of this License, Derivative Works shall not include works that remain
+      separable from, or merely link (or bind by name) to the interfaces of,
+      the Work and Derivative Works thereof.
+
+      "Contribution" shall mean any work of authorship, including
+      the original version of the Work and any modifications or additions
+      to that Work or Derivative Works thereof, that is intentionally
+      submitted to Licensor for inclusion in the Work by the copyright owner
+      or by an individual or Legal Entity authorized to submit on behalf of
+      the copyright owner. For the purposes of this definition, "submitted"
+      means any form of electronic, verbal, or written communication sent
+      to the Licensor or its representatives, including but not limited to
+      communication on electronic mailing lists, source code control systems,
+      and issue tracking systems that are managed by, or on behalf of, the
+      Licensor for the purpose of discussing and improving the Work, but
+      excluding communication that is conspicuously marked or otherwise
+      designated in writing by the copyright owner as "Not a Contribution."
+
+      "Contributor" shall mean Licensor and any individual or Legal Entity
+      on behalf of whom a Contribution has been received by Licensor and
+      subsequently incorporated within the Work.
+
+   2. Grant of Copyright License. Subject to the terms and conditions of
+      this License, each Contributor hereby grants to You a perpetual,
+      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
+      copyright license to reproduce, prepare Derivative Works of,
+      publicly display, publicly perform, sublicense, and distribute the
+      Work and such Derivative Works in Source or Object form.
+
+   3. Grant of Patent License. Subject to the terms and conditions of
+      this License, each Contributor hereby grants to You a perpetual,
+      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
+      (except as stated in this section) patent license to make, have made,
+      use, offer to sell, sell, import, and otherwise transfer the Work,
+      where such license applies only to those patent claims licensable
+      by such Contributor that are necessarily infringed by their
+      Contribution(s) alone or by combination of their Contribution(s)
+      with the Work to which such Contribution(s) was submitted. If You
+      institute patent litigation against any entity (including a
+      cross-claim or counterclaim in a lawsuit) alleging that the Work
+      or a Contribution incorporated within the Work constitutes direct
+      or contributory patent infringement, then any patent licenses
+      granted to You under this License for that Work shall terminate
+      as of the date such litigation is filed.
+
+   4. Redistribution. You may reproduce and distribute copies of the
+      Work or Derivative Works thereof in any medium, with or without
+      modifications, and in Source or Object form, provided that You
+      meet the following conditions:
+
+      (a) You must give any other recipients of the Work or
+          Derivative Works a copy of this License; and
+
+      (b) You must cause any modified files to carry prominent notices
+          stating that You changed the files; and
+
+      (c) You must retain, in the Source form of any Derivative Works
+          that You distribute, all copyright, patent, trademark, and
+          attribution notices from the Source form of the Work,
+          excluding those notices that do not pertain to any part of
+          the Derivative Works; and
+
+      (d) If the Work includes a "NOTICE" text file as part of its
+          distribution, then any Derivative Works that You distribute must
+          include a readable copy of the attribution notices contained
+          within such NOTICE file, excluding those notices that do not
+          pertain to any part of the Derivative Works, in at least one
+          of the following places: within a NOTICE text file distributed
+          as part of the Derivative Works; within the Source form or
+          documentation, if provided along with the Derivative Works; or,
+          within a display generated by the Derivative Works, if and
+          wherever such third-party notices normally appear. The contents
+          of the NOTICE file are for informational purposes only and
+          do not modify the License. You may add Your own attribution
+          notices within Derivative Works that You distribute, alongside
+          or as an addendum to the NOTICE text from the Work, provided
+          that such additional attribution notices cannot be construed
+          as modifying the License.
+
+      You may add Your own copyright statement to Your modifications and
+      may provide additional or different license terms and conditions
+      for use, reproduction, or distribution of Your modifications, or
+      for any such Derivative Works as a whole, provided Your use,
+      reproduction, and distribution of the Work otherwise complies with
+      the conditions stated in this License.
+
+   5. Submission of Contributions. Unless You explicitly state otherwise,
+      any Contribution intentionally submitted for inclusion in the Work
+      by You to the Licensor shall be under the terms and conditions of
+      this License, without any additional terms or conditions.
+      Notwithstanding the above, nothing herein shall supersede or modify
+      the terms of any separate license agreement you may have executed
+      with Licensor regarding such Contributions.
+
+   6. Trademarks. This License does not grant permission to use the trade
+      names, trademarks, service marks, or product names of the Licensor,
+      except as required for reasonable and customary use in describing the
+      origin of the Work and reproducing the content of the NOTICE file.
+
+   7. Disclaimer of Warranty. Unless required by applicable law or
+      agreed to in writing, Licensor provides the Work (and each
+      Contributor provides its Contributions) on an "AS IS" BASIS,
+      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+      implied, including, without limitation, any warranties or conditions
+      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
+      PARTICULAR PURPOSE. You are solely responsible for determining the
+      appropriateness of using or redistributing the Work and assume any
+      risks associated with Your exercise of permissions under this License.
+
+   8. Limitation of Liability. In no event and under no legal theory,
+      whether in tort (including negligence), contract, or otherwise,
+      unless required by applicable law (such as deliberate and grossly
+      negligent acts) or agreed to in writing, shall any Contributor be
+      liable to You for damages, including any direct, indirect, special,
+      incidental, or consequential damages of any character arising as a
+      result of this License or out of the use or inability to use the
+      Work (including but not limited to damages for loss of goodwill,
+      work stoppage, computer failure or malfunction, or any and all
+      other commercial damages or losses), even if such Contributor
+      has been advised of the possibility of such damages.
+
+   9. Accepting Warranty or Additional Liability. While redistributing
+      the Work or Derivative Works thereof, You may choose to offer,
+      and charge a fee for, acceptance of support, warranty, indemnity,
+      or other liability obligations and/or rights consistent with this
+      License. However, in accepting such obligations, You may act only
+      on Your own behalf and on Your sole responsibility, not on behalf
+      of any other Contributor, and only if You agree to indemnify,
+      defend, and hold each Contributor harmless for any liability
+      incurred by, or claims asserted against, such Contributor by reason
+      of your accepting any such warranty or additional liability.
+
+   END OF TERMS AND CONDITIONS
+
+   APPENDIX: How to apply the Apache License to your work.
+
+      To apply the Apache License to your work, attach the following
+      boilerplate notice, with the fields enclosed by brackets "[]"
+      replaced with your own identifying information. (Don't include
+      the brackets!)  The text should be enclosed in the appropriate
+      comment syntax for the file format. We also recommend that a
+      file or class name and description of purpose be included on the
+      same "printed page" as the copyright notice for easier
+      identification within third-party archives.
+
+   Copyright [yyyy] [name of copyright owner]
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/resources/META-INF/NOTICE.txt
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/resources/META-INF/NOTICE.txt b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/NOTICE.txt
new file mode 100644
index 0000000..2e215bf
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/NOTICE.txt
@@ -0,0 +1,11 @@
+   =========================================================================
+   ==  NOTICE file corresponding to the section 4 d of                    ==
+   ==  the Apache License, Version 2.0,                                   ==
+   ==  in this case for the Apache Camel distribution.                    ==
+   =========================================================================
+
+   This product includes software developed by
+   The Apache Software Foundation (http://www.apache.org/).
+
+   Please read the different LICENSE files present in the licenses directory of
+   this distribution.

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/resources/META-INF/beans.xml
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/resources/META-INF/beans.xml b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/beans.xml
new file mode 100644
index 0000000..112d56d
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/resources/META-INF/beans.xml
@@ -0,0 +1,18 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!--
+  Licensed to the Apache Software Foundation (ASF) under one or more
+  contributor license agreements.  See the NOTICE file distributed with
+  this work for additional information regarding copyright ownership.
+  The ASF licenses this file to You under the Apache License, Version 2.0
+  (the "License"); you may not use this file except in compliance with
+  the License.  You may obtain a copy of the License at
+
+  http://www.apache.org/licenses/LICENSE-2.0
+
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+-->
+<beans/>
\ No newline at end of file

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service1/src/main/resources/log4j.properties
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service1/src/main/resources/log4j.properties b/examples/camel-example-zipkin/service1/src/main/resources/log4j.properties
new file mode 100644
index 0000000..d04caeb
--- /dev/null
+++ b/examples/camel-example-zipkin/service1/src/main/resources/log4j.properties
@@ -0,0 +1,28 @@
+## ------------------------------------------------------------------------
+## Licensed to the Apache Software Foundation (ASF) under one or more
+## contributor license agreements.  See the NOTICE file distributed with
+## this work for additional information regarding copyright ownership.
+## The ASF licenses this file to You under the Apache License, Version 2.0
+## (the "License"); you may not use this file except in compliance with
+## the License.  You may obtain a copy of the License at
+##
+## http://www.apache.org/licenses/LICENSE-2.0
+##
+## Unless required by applicable law or agreed to in writing, software
+## distributed under the License is distributed on an "AS IS" BASIS,
+## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+## See the License for the specific language governing permissions and
+## limitations under the License.
+## ------------------------------------------------------------------------
+
+#
+# The logging properties used
+#
+log4j.rootLogger=INFO, stdout
+
+log4j.logger.org.apache.camel.zipkin=DEBUG
+
+# CONSOLE appender not used by default
+log4j.appender.stdout=org.apache.log4j.ConsoleAppender
+log4j.appender.stdout.layout=org.apache.log4j.PatternLayout
+log4j.appender.stdout.layout.ConversionPattern=%d [%-15.15t] %-5p %-30.30c{1} - %m%n

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service2/pom.xml
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service2/pom.xml b/examples/camel-example-zipkin/service2/pom.xml
new file mode 100644
index 0000000..d037c9a
--- /dev/null
+++ b/examples/camel-example-zipkin/service2/pom.xml
@@ -0,0 +1,90 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!--
+    Licensed to the Apache Software Foundation (ASF) under one or more
+    contributor license agreements. See the NOTICE file distributed with
+    this work for additional information regarding copyright ownership.
+    The ASF licenses this file to You under the Apache License, Version
+    2.0 (the "License"); you may not use this file except in compliance
+    with the License. You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0 Unless required by
+    applicable law or agreed to in writing, software distributed under the
+    License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
+    CONDITIONS OF ANY KIND, either express or implied. See the License for
+    the specific language governing permissions and limitations under the
+    License.
+-->
+<project xmlns="http://maven.apache.org/POM/4.0.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/maven-v4_0_0.xsd">
+
+  <modelVersion>4.0.0</modelVersion>
+
+ <parent>
+    <groupId>org.apache.camel</groupId>
+    <artifactId>camel-example-zipkin</artifactId>
+    <version>2.18-SNAPSHOT</version>
+  </parent>
+  
+  <artifactId>camel-example-zipkin-service2</artifactId>
+  <name>Camel :: Example :: Zipkin :: Service 2</name>
+  <description>An example showing how to trace incoming and outgoing messages from Camel with Zipkin</description>
+
+  <properties>
+    <spring.boot-version>${spring-boot-version}</spring.boot-version>
+  </properties>
+
+  <!-- import Spring-Boot and Camel BOM -->
+  <dependencyManagement>
+    <dependencies>
+      <dependency>
+        <groupId>org.springframework.boot</groupId>
+        <artifactId>spring-boot-dependencies</artifactId>
+        <version>${spring.boot-version}</version>
+        <type>pom</type>
+        <scope>import</scope>
+      </dependency>
+      <dependency>
+        <groupId>org.apache.camel</groupId>
+        <artifactId>camel-parent</artifactId>
+        <version>${project.version}</version>
+        <type>pom</type>
+        <scope>import</scope>
+      </dependency>
+    </dependencies>
+  </dependencyManagement>
+
+  <dependencies>
+
+    <!-- spring-boot -->
+    <dependency>
+      <groupId>org.springframework.boot</groupId>
+      <artifactId>spring-boot-starter</artifactId>
+    </dependency>
+
+    <!-- camel -->
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-spring-boot-starter</artifactId>
+    </dependency>
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-zipkin-starter</artifactId>
+    </dependency>
+    <dependency>
+      <groupId>org.apache.camel</groupId>
+      <artifactId>camel-jetty</artifactId>
+    </dependency>
+
+  </dependencies>
+
+  <build>
+    <plugins>
+      <plugin>
+        <groupId>org.springframework.boot</groupId>
+        <artifactId>spring-boot-maven-plugin</artifactId>
+        <version>${spring-boot-version}</version>
+      </plugin>
+    </plugins>
+  </build>
+
+</project>

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Application.java
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Application.java b/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Application.java
new file mode 100644
index 0000000..6fe12a8
--- /dev/null
+++ b/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Application.java
@@ -0,0 +1,42 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package sample.camel;
+
+import org.apache.camel.zipkin.starter.CamelZipkin;
+import org.springframework.boot.SpringApplication;
+import org.springframework.boot.autoconfigure.SpringBootApplication;
+
+//CHECKSTYLE:OFF
+/**
+ * A Spring Boot application that starts the Camel Zipkin application.
+ * <p/>
+ * Notice we use the `@CamelZipkin` annotation to enable Camel with Zipkin.
+ * The configuration of Zipkin is in the <tt>application.properties</tt> file.
+ */
+@SpringBootApplication
+@CamelZipkin
+public class Service2Application {
+
+    /**
+     * A main method to start this application.
+     */
+    public static void main(String[] args) {
+        SpringApplication.run(Service2Application.class, args);
+    }
+
+}
+//CHECKSTYLE:ON

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Route.java
----------------------------------------------------------------------
diff --git a/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Route.java b/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Route.java
new file mode 100644
index 0000000..e95582b
--- /dev/null
+++ b/examples/camel-example-zipkin/service2/src/main/java/sample/camel/Service2Route.java
@@ -0,0 +1,33 @@
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+package sample.camel;
+
+import org.apache.camel.builder.RouteBuilder;
+import org.springframework.stereotype.Component;
+
+@Component
+public class Service2Route extends RouteBuilder {
+
+    @Override
+    public void configure() throws Exception {
+        from("jetty:http://0.0.0.0:{{jetty.port}}/service2").routeId("service2")
+            .convertBodyTo(String.class)
+            .delay(simple("${random(1000,2000)}"))
+            .transform(simple("Hello: ${body}"));
+    }
+
+}
diff --git a/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinConfigurationProperties.java b/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinConfigurationProperties.java
index 18b95e8..a342a69 100644
--- a/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinConfigurationProperties.java
+++ b/components/camel-zipkin-starter/src/main/java/org/apache/camel/zipkin/starter/ZipkinConfigurationProperties.java
@@ -59,10 +59,16 @@ public class ZipkinConfigurationProperties {
     private Set<String> excludePatterns;
 
     /**
      * Sets service mapping(s) that matches Camel events to the given zipkin service name.
      * The key is the pattern, the value is the service name.
      */
-    private Map<String, String> serviceMappings;
+    private Map<String, String> clientServiceMappings;
+
+    /**
+     * Sets server service mapping(s) that matches Camel events to the given zipkin service name.
+     * The key is the pattern, the value is the service name.
+     */
+    private Map<String, String> serverServiceMappings;
 
     // Getters & setters
 
@@ -114,11 +120,19 @@ public class ZipkinConfigurationProperties {
         this.excludePatterns = excludePatterns;
     }
 
-    public Map<String, String> getServiceMappings() {
-        return serviceMappings;
+    public Map<String, String> getClientServiceMappings() {
+        return clientServiceMappings;
+    }
+
+    public void setClientServiceMappings(Map<String, String> clientServiceMappings) {
+        this.clientServiceMappings = clientServiceMappings;
+    }
+
+    public Map<String, String> getServerServiceMappings() {
+        return serverServiceMappings;
     }
 
-    public void setServiceMappings(Map<String, String> serviceMappings) {
-        this.serviceMappings = serviceMappings;
+    public void setServerServiceMappings(Map<String, String> serverServiceMappings) {
+        this.serverServiceMappings = serverServiceMappings;
     }
 }

http://git-wip-us.apache.org/repos/asf/camel/blob/4dc5b5e4/components/camel-zipkin/pom.xml
----------------------------------------------------------------------
@@ -59,10 +59,16 @@ public class ZipkinConfigurationProperties {
 
     /**
-     * Sets service mapping(s) that matches Camel events to the given zipkin service name.
+     * Sets client service mapping(s) that matches Camel events to the given zipkin service name.
      * The key is the pattern, the value is the service name.
      */